import "JackdawPlugins.Libs.Windows"
import "JackdawPlugins.Libs.Quickslots"
import "JackdawPlugins.Libs.Common"

--[[**NewQBar**
	A new version of the NewBar library.
	This only uses a window if it needs to - if you have already built a window call QBar rather than NewQbar.
	
	This avoids creating sub-windows.
	
	It is assumed that once the Bar has been created the calling application will ensure the variables remain consitent.
	
	It consists of two elements;
		*NewQBar(name,{options}) creates a full bar, along with parent window.
		*QBar(name,parent window) is a control class that consists of the Quickslots fitted to that window.
		
		example:
		To create a complete bar;											geoff={} NewQBar(geoff,{x=0,y=0,sx=10,sy=5})
		To assign quickslots to a window called "hiram"		whereyouwanttostorethequickslots=QBar(hiram)
		self						Table holding the information
		self.options			Data. The options for the bar.
		self.Window			Object. The parent window
		self.Quickslot[i]	Object. The Quickslots.	NOTE: icon size and gap are stored here also.
		
		self.shortcut[i]	Data. The shortcuts from the Quickslots in .Data,.Type format
		
		convention: Data is all lower case. Objects have an uppercase initial.]]--
	
function NewQBar(self,options)
--This bar is an entity in itself. IT SHOULD NOT/CANNOT BE PARENTED TO ANOTHER WINDOW.
	self.options=options
	local x=options.x or 0
	local y=options.y or 0
	local sx=options.sx or 1
	local sy=options.sy or 1
	local colour=options.colour or Turbine.UI.Color(0.2,1,1,0)
	local name=options.name or "Bar"
	local dragbar=options.dragbar or false
	local size=options.size or 35
	local gap=options.gap or 0
	local pad=options.pad or 0

	local width=sx*(size+gap)+gap+2*pad
	local height=sy*(size+gap)+gap+2*pad

	--Create a window to hold the bar
	self.Window=NewWindow {x=x,y=y,width=width,height=height, colour=colour}

	--Assign Dragbar to window
	if dragbar then import "Deusdictum.UI.Dragbar" self.Window.DragBar = Deusdictum.UI.DragBar( self.Window, name); end

	--assign the Quickslots
	self.Quickslot=QBar(self.Window,size,gap,pad)	

	--Register to record position if moved.
	self.Window.PositionChanged = function( sender, args ) 
		if self.Window~=nil then self.options.x,self.options.y=self.Window:GetPosition() end
	end
end

--Create a Quickslot bar - INDEPENDANT OF ITS WINDOW NOW!
--This creates other complications, but I am fine with this...
QBar=class(Turbine.UI.Control)

function QBar:Constructor(parent,size,gap,pad)
--this will be called by ---.Quickslot, which is what self will refer to.
size=size or 35
gap=gap or 0
pad=pad or 0

self.size=size
self.gap=gap
self.pad=pad

local width, height=parent:GetSize() -- Get the pixel size of the frame
local sx=math.floor((width-2*pad)/(size+gap)) --convert to icons
local sy=math.floor((height-2*pad)/(size+gap))


--Build the Quickslots
	if (sx>1) and (sy>1) then	--a 2d bar
			local a=1 --a tracks the index
			for i = 1,sx do
				for j=1,sy do
					local x=pad+(size+gap)*(i-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
					local y=pad+(size+gap)*(j-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
					self [a]=NewQuickslot {x=x,y=y,gap=1,parent=parent,size=size} 
					a=a+1
				end
			end
		elseif sx>1 then--a wide bar
			for i = 1,sx do
				local x=pad+(size+gap)*(i-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				self [i]=NewQuickslot {x=x,y=y,gap=1,parent=parent,size=size} 
			end
		elseif sy>1 then-- a long bar
			for j=1,sy do
				local x=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+(size+gap)*(j-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
				self [j]=NewQuickslot {x=x,y=y,gap=1,parent=parent,size=size} 
			end
		elseif sx*sy==1 then -- a single slot!
				local x=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				self [1]=NewQuickslot {x=x,y=y,gap=1,parent=parent,size=size} 
	end
--Register to change Quickslots if parents size is changed, and resize the dragbar
	parent.SizeChanged =function(sender,args)
		QBarSizeChanged(self,parent)
		if parent.DragBar~=nil then parent.DragBar:RecalculateSize() end
	end
end

--The parent window size has changed - so therefore so do I.
function QBarSizeChanged(self,parent)

local size=self.size or 35
local gap=self.gap or 0
local pad=self.pad or 0

local width, height=parent:GetSize() -- Get the pixel size of the frame
local sx=math.floor((width-2*pad)/(size+gap)) --convert to icons
local sy=math.floor((height-2*pad)/(size+gap))


	--store the current number of Quickslots
	local oldsize=#self
	--Add/subtract Quickslots
	--Concept - add/delete the Quickslots THEN sort out positions.
		--First, are we increasing or decreasing the number of Quickslots? Or does it stay the same?
		--calaculate the number of slots in the new bar
			local newsize=sx*sy
			local change=newsize-oldsize
			--Note: if a+b=0 then nothing happens...
			if change<0 then
				--We delete some Quickslots
				for i=newsize+1,oldsize do
					self[newsize+1]:SetVisible(false)
					table.remove(self,newsize+1) 
				end
			elseif change>0 then--increase
				
				--We add some Quickslots, stick them all at (0,0) for now.
				local c=oldsize
				for i=oldsize+1,newsize do
					c=c+1
						self [c]=NewQuickslot {x=0,y=0,gap=gap,parent=parent}
				end
			end
		--Second we re-position the Quickslots
		if (sx>1) and (sy>1) then	--a 2d bar
			local c=1 --c tracks the index
			for i = 1,sx do
				for j=1,sy do
					local x=pad+(size+gap)*(i-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
					local y=pad+(size+gap)*(j-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
					self [c]:SetPosition(x,y)
					self [c]:SetVisible(true)
					c=c+1
				end
			end
		elseif sx>1 then--a wide bar
			for i = 1,sx do
				local x=pad+(size+gap)*(i-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				self[i]:SetPosition(x,y)
			end
		elseif sy>1 then-- a long bar
			for j=1,sy do
				local x=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+(size+gap)*(j-1)+gap-2	--The -2 is arbitrary but seems to centre the slots better.
				self [j]:SetPosition(x,y)
			end
		elseif sx*sy==1 then -- a single slot!
				local x=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				local y=pad+gap-2								--The -2 is arbitrary but seems to centre the slots better.
				self [1]:SetPosition(x,y)
		end			
end

--Returns a table of the shortcuts in the quickslots.
function QBar:StoreSlots()
		destination={}
		for i = 1,#self do
			local shortcut=self[i]:GetShortcut()
			destination[i]={}
			destination[i].Data=shortcut:GetData()
			destination[i].Type=shortcut:GetType()
		end
		return destination
end

--Load shortcuts from the source table.
function QBar:LoadSlots(source)
	for i=1,#self do
	if source~=nil then PSetShortcut(self[i],Turbine.UI.Lotro.Shortcut( source[i].Type,source[i].Data )) end
	end
end